/*
 * udpmouse-send: send mouse input_event through a UDP socket
 *
 * Copyright (c) 2004 Daniele Bellucci <belch@linux.it>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */



#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include <linux/input.h>
#include <asm/bitops.h>


/* mainly copied from include/asm-i386/types.h , include/linux/input.h */
#define BITS_PER_LONG 32
#define NBITS(x) ((((x)-1)/BITS_PER_LONG)+1)


static int evdev_open_mouseintf(char *name)
{
    int fd;
    unsigned long evbit[NBITS(EV_MAX)];
    unsigned long keybit[NBITS(KEY_MAX)];
    unsigned long relbit[NBITS(REL_MAX)];

    memset(evbit, 0, sizeof(evbit));
    memset(keybit, 0, sizeof(keybit));
    memset(relbit, 0, sizeof(keybit));

    /* open the interface */
    fd = open(name, O_RDONLY);
    if (fd == -1)
	return -1;
    

    /* take the event types */
    if (ioctl(fd, EVIOCGBIT(0, EV_MAX), evbit) < 0) {
	close(fd);
	return -1;
    }

    if (!test_bit(EV_REL, evbit) || !test_bit(EV_KEY, evbit)) {
	close(fd);
	return -1;
    }


    /* take relbit */
    if (ioctl(fd, EVIOCGBIT(EV_REL, sizeof(relbit)), relbit) < 0) {
	close(fd);
	return -1;
    }
    if (!test_bit(REL_X, relbit) || !test_bit(REL_Y, relbit)) {
	close(fd);
	return -1;
    }


    /* take the keybit */
    if (ioctl(fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit) < 0) {
	close(fd);
	return -1;
    }
    if (!test_bit(BTN_LEFT, keybit) || !test_bit(BTN_RIGHT, keybit) ||
	!test_bit(BTN_MIDDLE, keybit))
    {
	close(fd);
	return -1;
    }

    return fd;
}




int main(int argc, char **argv) 
{
    int sock, fd, retval, size, i;
    struct sockaddr_in addr_peer;
    struct input_event event;
    
    if (argc != 4) {
	fprintf(stderr, "Usage: \"%s <server_ip> <port> <evdev_intf>\"\n", 
		argv[0]);
	exit(1);
    }

    /* open the evdev interface */
    retval = evdev_open_mouseintf(argv[3]);
    if (retval == -1) {
	fprintf(stderr, "%s: evdev_open_mouseint(\"%s\"): ", argv[0], argv[3]);
	
	if (errno) {
	    fprintf(stderr, "%s\n",  strerror(errno));
	} else {
	    fprintf(stderr, "doesn't look a valid mouse interface :( \n");
	}
	exit(1);
    }

    fd = retval;

    /* Create the UDP socket */
    if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	fprintf(stderr, "%s: socket(): %s\n", argv[0], strerror(errno));
	exit(1);
    }

    /* Initialize the addr_peer */
    memset(&addr_peer, 0, sizeof(addr_peer));      
    addr_peer.sin_family = AF_INET;                  
    addr_peer.sin_addr.s_addr = inet_addr(argv[1]);  
    addr_peer.sin_port = htons(atoi(argv[2]));       


    while(1) {
	/* Take an input_event from evdev interface */
	retval = read(fd, &event, sizeof(struct input_event));
	if (retval == -1) {
	    fprintf(stderr, "%s: read(<input_event>): %s\n", argv[0], 
		    strerror(errno));
	    exit(1);
	}

	/* send input_event to the peer */
	retval = sendto(sock, &event, sizeof(struct input_event), 0,
			(struct sockaddr *)&addr_peer, sizeof(addr_peer));

	if (retval == -1) {
	    fprintf(stderr, "%s: sendto(<%s>): %s\n", argv[0], 
		    argv[1], strerror(errno));
	    exit(1);
	}
    }
}
 
